<?php

namespace TDevAgency\CmsSitemap\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Response;
use Mcamara\LaravelLocalization\Facades\LaravelLocalization;
use SimpleXMLElement;
use TDevAgency\Cms\Models\BaseModels\Post;
use TDevAgency\Cms\Models\BaseModels\PostLocalization;

class SitemapController extends Controller
{
    public function index()
    {
        $xml = new SimpleXMLElement('<?xml version="1.0"  encoding="utf-8"?><sitemapindex/>');
        $xml->addAttribute('xmlns:xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
        $xml->addAttribute(
            'xsi:xsi:schemaLocation',
            'http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/siteindex.xsd'
        );
        $xml->addAttribute('xmlns', 'http://www.sitemaps.org/schemas/sitemap/0.9');

        $types = Config::get('cms-sitemap');

        foreach ($types as $type => $config) {
            $sitemap = $xml->addChild('sitemap');
            $sitemap->addChild('loc', route('sitemap.post_type', $type));
        }

        return Response::make($xml->asXML())
            ->header('Content-Type', 'text/xml');
    }

    public function single($postType)
    {
        if (!Config::has('cms-sitemap.' . $postType)) {
            $this->notFound();
        }
        $config = Config::get('cms-sitemap.' . $postType);

        $xml = new SimpleXMLElement('<?xml version="1.0" encoding="utf-8"?><urlset/>');
        $xml->addAttribute('xmlns', 'http://www.sitemaps.org/schemas/sitemap/0.9');
        $xml->addAttribute('xmlns:xmlns:xhtml', 'http://www.w3.org/1999/xhtml');

        $posts = $this->getPosts($postType, $config);

        if ($posts->isNotEmpty()) {
            $posts->each(function (Post $post) use (&$xml, $config) {
                $url = $xml->addChild('url');
                $url->addChild(
                    'loc',
                    LaravelLocalization::getLocalizedURL(LaravelLocalization::getDefaultLocale(), $post->slug)
                );
                if ($post->slug === '/') {
//                    $url->addChild('changefreq', 'hourly');
                } else {
//                    $url->addChild('changefreq', $config['changefreq']);
                    $url->addChild('lastmod', $post->updated_at->toW3cString());
//                    $url->addChild('priority', $config['priority']);

                }


                $post->localizations->each(function (PostLocalization $postLocalization) use ($post, &$url) {
                    $link = $url->addChild('xhtml:xhtml:link ');
                    $link->addAttribute('rel', 'alternate');
                    $link->addAttribute('hreflang', $postLocalization->lang);
                    $link->addAttribute(
                        'href',
                        LaravelLocalization::getLocalizedURL($postLocalization->lang, $post->slug)
                    );
                });

//                $url->addChild('loc', route('page.home'));
                //        $url->addChild('lastmod', $sitemapModel->getHomeUpdate());
//                $url->addChild('changefreq', 'hourly');


            });
        }

        return Response::make($xml->asXML())
            ->header('Content-Type', 'text/xml');
    }

    public function notFound(): void
    {
        abort(404);
    }

    protected function getPosts($postType, $config)
    {
        return Post::where('post_type', $config['post_type'])->where('post_status', 1)
            ->with('localizations')
            ->get();
    }
}
